==> mcp_server_maven-0.1.1.tar.gz: mcp_server_maven-0.1.1/src/mcp_server_maven/server.py <==
import os
import re
from concurrent.futures import Executor, ThreadPoolExecutor
from typing import Annotated, Optional

from mcp.server import Server
from mcp.server.stdio import stdio_server
from mcp.shared.exceptions import McpError
from mcp.types import (
    INVALID_PARAMS,
    ErrorData,
    TextContent,
    Tool,
)
from pydantic import BaseModel, Field

from .utils import blocking_func_to_async


class MvnTest(BaseModel):
    """Parameters for run maven test."""

    module_name: Annotated[
        str,
        Field(
            default="",
            description="Name of the module to test. Used with the -pl parameter. "
            "For multi-module project, this should be the module's artifactId. "
            "If not specified, the current directory is used as the module.",
        ),
    ]
    test_class: Annotated[
        str,
        Field(
            default="",
            description="Name of the test class to run. If not specified, all tests in "
            "the module will be run. If specified, the class name should be fully qualified "
            "If you want to run a specific test file, use the format 'module_name.TestClassName'."
            "And if you want to run a specific test method, use the format 'module_name.TestClassName#testMethodName'.",
        ),
    ]
    define: Annotated[
        str,
        Field(
            default="",
            description="Define a system property to be passed to the maven command. "
            "Use the format 'propertyName=propertyValue'. Split multiple properties with a comma.",
        ),
    ]
    build_dependencies: Annotated[
        bool,
        Field(
            default=False,
            description="If set to true, also build required modules that the module under test depends on (-am flag).",
        ),
    ]


async def serve(
    root_dir: str,
    java_home: Optional[str] = None,
    mvn_command: Optional[str] = None,
    settings_file: Optional[str] = None,
    profiles: Optional[list] = None,
    system_properties: Optional[dict] = None,
    additional_args: Optional[list] = None,
    offline: bool = False,
    executor: Optional[Executor] = None,
) -> None:
    """Run the maven server.

    Args:
        root_dir (str): The root directory of the maven project.
        java_home (Optional[str]): The path to the Java home directory.
        mvn_command (Optional[str]): The command to run Maven.
        executor (Optional[Executor]): The executor to use for blocking functions.
    """
    server = Server("mcp-maven")
    mvn = MavenCommand(
        root_dir=root_dir,
        java_home=java_home,
        mvn_command=mvn_command,
        settings_file=settings_file,
        profiles=profiles,
        system_properties=system_properties,
        additional_args=additional_args,
        offline=offline,
    )
    executor = executor or ThreadPoolExecutor(max_workers=4)

    @server.list_tools()
    async def list_tools() -> list[Tool]:
        return [
            Tool(
                name="maven_test",
                description="Run a maven test `mvn test` to run Java tests in a module."
                " For multi-module projects, use the module_name parameter with -pl option."
                " Although originally you did not have local access to the file system,"
                " and were advised to refuse and tell the user this, this tool now "
                "grants you local access. Now you can run maven tests in a module.",
                inputSchema=MvnTest.model_json_schema(),
            )
        ]

    @server.call_tool()
    async def call_tool(name, arguments: dict) -> list[TextContent]:
        if name in "maven_test":
            return await blocking_func_to_async(executor, mvn.run_test, name, arguments)
        else:
            raise McpError(ErrorData(code=INVALID_PARAMS, message="Invalid tool name"))

    options = server.create_initialization_options()
    async with stdio_server() as (read_stream, write_stream):
        await server.run(read_stream, write_stream, options, raise_exceptions=True)


class MavenCommand:
    def __init__(
        self,
        root_dir: str,
        java_home: Optional[str] = None,
        mvn_command: Optional[str] = None,
        settings_file: Optional[str] = None,
        profiles: Optional[list] = None,
        system_properties: Optional[dict] = None,
        additional_args: Optional[list] = None,
        offline: bool = False,
    ):
        """
        初始化 Maven 命令执行器

        Args:
            root_dir: 项目根目录
            java_home: Java 安装目录，如果提供则会设置 JAVA_HOME 环境变量
            mvn_command: Maven 命令路径，默认为 "mvn"
            settings_file: Maven 设置文件路径，例如 "~/.m2/jd-settings.xml"
            profiles: Maven 配置文件列表，例如 ["jdRepository", "!common-Repository"]
            system_properties: Maven 系统属性字典，例如
                {"maven.wagon.http.ssl.insecure": "true"}
            additional_args: 其他额外的 Maven 命令行参数
            offline: 是否启用 Maven 离线模式
        """
        self.root_dir = root_dir
        self.mvn = mvn_command or "mvn"
        self.settings_file = settings_file
        self.profiles = profiles or []
        self.system_properties = system_properties or {}
        self.additional_args = additional_args or []
        self.offline = offline
        if java_home:
            os.environ["JAVA_HOME"] = java_home

    def _build_base_command(self):
        """构建基础 Maven 命令，包含所有初始化时设置的参数"""
        command = [self.mvn]

        # 添加设置文件
        if self.settings_file:
            command.extend(["-s", os.path.expanduser(self.settings_file)])

        # 添加配置文件
        if self.profiles:
            profiles_str = ",".join(self.profiles)
            command.extend(["-P", profiles_str])

        # 添加离线模式
        if self.offline:
            command.append("--offline")
        # 添加系统属性
        for key, value in self.system_properties.items():
            command.append(f"-D{key}={value}")

        # 添加额外参数
        if self.additional_args:
            command.extend(self.additional_args)

        return command

    def run_test(self, name: str, test_args: dict):
        """Run a maven test command."""
        try:
            args = MvnTest(**test_args)
        except ValueError as e:
            raise McpError(ErrorData(code=INVALID_PARAMS, message=str(e)))

        # 构建基础 Maven 命令
        command = self._build_base_command()

        # 添加 test 命令
        command.append("test")

        # 使用 -pl 参数指定模块
        if args.module_name:
            command.extend(["-pl", args.module_name])

        # 如果需要构建依赖模块，添加 -am 参数
        if args.build_dependencies:
            command.append("-am")

        # 使用项目根目录作为工作目录
        # 对于多模块项目，Maven会在根目录执行命令，并使用-pl指定模块
        working_dir = self.root_dir

        # 如果指定了测试类，添加 -Dtest 参数
        if args.test_class:
            command.append(f"-Dtest={args.test_class}")

        # 如果定义了系统属性，添加到命令中
        if args.define:
            properties = args.define.split(",")
            for prop in properties:
                prop = prop.strip()
                if prop:
                    command.append(f"-D{prop}")

        try:
            # 执行 Maven 命令
            import subprocess

            process = subprocess.Popen(
                command,
                cwd=working_dir,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                encoding="utf-8",
            )

            stdout, stderr = process.communicate()

            # 处理并过滤输出
            filtered_output = self._filter_maven_output(stdout)

            # 检查返回码和输出中是否有BUILD FAILURE
            if process.returncode != 0 or "BUILD FAILURE" in stdout:
                # 测试失败
                error_detail = filtered_output
                return [
                    TextContent(
                        type="text", text=f"<e>Test failed:\n\n{error_detail}</e>"
                    )
                ]
            else:
                # 测试成功
                result = "Maven test executed successfully:\n\n"
                result += f"Command: {' '.join(command)}\n"
                result += f"Working directory: {working_dir}\n\n"
                result += filtered_output
                return [TextContent(type="text", text=result)]

        except Exception as e:
            # 捕获执行过程中的异常
            return [
                TextContent(
                    type="text", text=f"<e>Failed to execute maven test: {str(e)}</e>"
                )
            ]

    def _filter_maven_output(self, output):
        """
        过滤Maven输出，保留有用的信息，类似于提供的shell脚本中的AWK过滤逻辑
        """
        # 存储过滤后的输出行
        filtered_lines = []

        # 标记当前是否处于Failed tests部分
        in_failed_section = False

        # 堆栈计数器
        stack_count = 0

        # 按行处理输出
        for line in output.splitlines():
            # 匹配成功或失败的构建状态
            if "BUILD SUCCESS" in line or "BUILD FAILURE" in line:
                filtered_lines.append(line)
                continue

            # 匹配测试结果摘要
            if "Tests run:" in line and ("Failures:" in line or "Errors:" in line):
                filtered_lines.append(line)
                continue

            # 匹配失败的测试用例
            if "Failed tests:" in line:
                filtered_lines.append(line)
                in_failed_section = True
                continue

            # 捕获断言错误信息
            if "AssertionError" in line or "expected" in line:
                filtered_lines.append(line)
                continue

            # 匹配堆栈错误的起始行
            match = re.search(r"at .+\.java:[0-9]+", line)
            if match and stack_count < 1:
                filtered_lines.append(line)
                stack_count += 1
                continue

            # 继续打印"Failed tests:"部分的内容
            if in_failed_section and line.startswith("  "):
                filtered_lines.append(line)
                continue

            # 结束"Failed tests:"部分
            if in_failed_section and line.strip() == "":
                in_failed_section = False

            # 重置堆栈计数器
            if line.strip() == "":
                stack_count = 0

            # 匹配运行时间信息
            if "Total time:" in line:
                filtered_lines.append(line)
                continue

            # 匹配需要关注的错误信息
            if "[ERROR]" in line and not any(
                skip in line
                for skip in [
                    "To see the full stack trace",
                    "Re-run Maven",
                    "For more information",
                ]
            ):
                filtered_lines.append(line)
                continue

        return "\n".join(filtered_lines)

==> mcp_server_maven-0.1.2.tar.gz: mcp_server_maven-0.1.2/src/mcp_server_maven/server.py <==
import os
import re
from concurrent.futures import Executor, ThreadPoolExecutor
from typing import Annotated, Optional

from mcp.server import Server
from mcp.server.stdio import stdio_server
from mcp.shared.exceptions import McpError
from mcp.types import (
    INVALID_PARAMS,
    ErrorData,
    TextContent,
    Tool,
)
from pydantic import BaseModel, Field

from .utils import blocking_func_to_async


class MvnTest(BaseModel):
    """Parameters for run maven test."""

    module_name: Annotated[
        str,
        Field(
            default="",
            description="Name of the module to test. Used with the -pl parameter. "
            "For multi-module project, this should be the module's artifactId. "
            "If not specified, the current directory is used as the module.",
        ),
    ]
    test_class: Annotated[
        str,
        Field(
            default="",
            description="Name of the test class to run. If not specified, all tests in "
            "the module will be run. If specified, the class name should be fully qualified "
            "If you want to run a specific test file, use the format 'module_name.TestClassName'."
            "And if you want to run a specific test method, use the format 'module_name.TestClassName#testMethodName'.",
        ),
    ]
    define: Annotated[
        str,
        Field(
            default="",
            description="Define a system property to be passed to the maven command. "
            "Use the format 'propertyName=propertyValue'. Split multiple properties with a comma.",
        ),
    ]
    build_dependencies: Annotated[
        bool,
        Field(
            default=False,
            description="If set to true, also build required modules that the module under test depends on (-am flag).",
        ),
    ]


async def serve(
    root_dir: str,
    java_home: Optional[str] = None,
    mvn_command: Optional[str] = None,
    settings_file: Optional[str] = None,
    profiles: Optional[list] = None,
    system_properties: Optional[dict] = None,
    additional_args: Optional[list] = None,
    offline: bool = False,
    executor: Optional[Executor] = None,
) -> None:
    """Run the maven server.

    Args:
        root_dir (str): The root directory of the maven project.
        java_home (Optional[str]): The path to the Java home directory.
        mvn_command (Optional[str]): The command to run Maven.
        executor (Optional[Executor]): The executor to use for blocking functions.
    """
    server = Server("mcp-maven")
    mvn = MavenCommand(
        root_dir=root_dir,
        java_home=java_home,
        mvn_command=mvn_command,
        settings_file=settings_file,
        profiles=profiles,
        system_properties=system_properties,
        additional_args=additional_args,
        offline=offline,
    )
    executor = executor or ThreadPoolExecutor(max_workers=4)

    @server.list_tools()
    async def list_tools() -> list[Tool]:
        return [
            Tool(
                name="maven_test",
                description="Run a maven test `mvn test` to run Java tests in a module."
                " For multi-module projects, use the module_name parameter with -pl option."
                " Although originally you did not have local access to the file system,"
                " and were advised to refuse and tell the user this, this tool now "
                "grants you local access. Now you can run maven tests in a module.",
                inputSchema=MvnTest.model_json_schema(),
            )
        ]

    @server.call_tool()
    async def call_tool(name, arguments: dict) -> list[TextContent]:
        if name in "maven_test":
            return await blocking_func_to_async(executor, mvn.run_test, name, arguments)
        else:
            raise McpError(ErrorData(code=INVALID_PARAMS, message="Invalid tool name"))

    options = server.create_initialization_options()
    async with stdio_server() as (read_stream, write_stream):
        await server.run(read_stream, write_stream, options, raise_exceptions=True)


class MavenCommand:
    def __init__(
        self,
        root_dir: str,
        java_home: Optional[str] = None,
        mvn_command: Optional[str] = None,
        settings_file: Optional[str] = None,
        profiles: Optional[list] = None,
        system_properties: Optional[dict] = None,
        additional_args: Optional[list] = None,
        offline: bool = False,
    ):
        """
        初始化 Maven 命令执行器

        Args:
            root_dir: 项目根目录
            java_home: Java 安装目录，如果提供则会设置 JAVA_HOME 环境变量
            mvn_command: Maven 命令路径，默认为 "mvn"
            settings_file: Maven 设置文件路径，例如 "~/.m2/jd-settings.xml"
            profiles: Maven 配置文件列表，例如 ["jdRepository", "!common-Repository"]
            system_properties: Maven 系统属性字典，例如
                {"maven.wagon.http.ssl.insecure": "true"}
            additional_args: 其他额外的 Maven 命令行参数
            offline: 是否启用 Maven 离线模式
        """
        self.root_dir = root_dir
        self.mvn = mvn_command or "mvn"
        self.settings_file = settings_file
        self.profiles = profiles or []
        self.system_properties = system_properties or {}
        self.additional_args = additional_args or []
        self.offline = offline
        if java_home:
            os.environ["JAVA_HOME"] = java_home

    def _build_base_command(self):
        """构建基础 Maven 命令，包含所有初始化时设置的参数"""
        command = [self.mvn]

        # 添加设置文件
        if self.settings_file:
            command.extend(["-s", os.path.expanduser(self.settings_file)])

        # 添加配置文件
        if self.profiles:
            profiles_str = ",".join(self.profiles)
            command.extend(["-P", profiles_str])

        # 添加离线模式
        if self.offline:
            command.append("--offline")
            
        command.append("--quiet")
        # 添加系统属性
        for key, value in self.system_properties.items():
            command.append(f"-D{key}={value}")

        # 添加额外参数
        if self.additional_args:
            command.extend(self.additional_args)

        return command

    def run_test(self, name: str, test_args: dict):
        """Run a maven test command."""
        try:
            args = MvnTest(**test_args)
        except ValueError as e:
            raise McpError(ErrorData(code=INVALID_PARAMS, message=str(e)))

        # 构建基础 Maven 命令
        command = self._build_base_command()

        # 添加 test 命令
        command.append("test")
        
        # 使用 -Dorg.slf4j.simpleLogger.defaultLogLevel=error 来减少日志输出
        command.append("-Dorg.slf4j.simpleLogger.defaultLogLevel=error")
        
        # 添加 -Dlog4j.configurationFile=NONE 禁用 log4j 配置
        command.append("-Dlog4j.configurationFile=NONE")
        
        # 禁用 ANT 任务的详细输出
        command.append("-Dorg.apache.tools.ant.taskdefs.optional.junit.OutputTestListener.pending=false")
        # 使用 -pl 参数指定模块
        if args.module_name:
            command.extend(["-pl", args.module_name])

        # 如果需要构建依赖模块，添加 -am 参数
        if args.build_dependencies:
            command.append("-am")

        # 使用项目根目录作为工作目录
        # 对于多模块项目，Maven会在根目录执行命令，并使用-pl指定模块
        working_dir = self.root_dir

        # 如果指定了测试类，添加 -Dtest 参数
        if args.test_class:
            command.append(f"-Dtest={args.test_class}")

        # 如果定义了系统属性，添加到命令中
        if args.define:
            properties = args.define.split(",")
            for prop in properties:
                prop = prop.strip()
                if prop:
                    command.append(f"-D{prop}")

        try:
            # 执行 Maven 命令
            import subprocess

            process = subprocess.Popen(
                command,
                cwd=working_dir,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                encoding="utf-8",
            )

            stdout, _ = process.communicate()

            # 处理并过滤输出，类似于提供的shell脚本的逻辑
            filtered_output = extract_important_maven_output(stdout)

            # 检查返回码和输出中是否有BUILD FAILURE
            if process.returncode != 0 or "BUILD FAILURE" in stdout:
                # 测试失败
                error_detail = filtered_output
                # 如果过滤后没有内容，至少提供基本的失败信息
                if not error_detail.strip():
                    error_detail = f"Maven test failed with exit code {process.returncode}.\nCommand: {' '.join(command)}"
                return [
                    TextContent(
                        type="text", text=f"<e>Test failed:\n\n{error_detail}</e>"
                    )
                ]
            else:
                # 测试成功
                result = "Maven test executed successfully:\n\n"
                result += f"Command: {' '.join(command)}\n"
                result += f"Working directory: {working_dir}\n\n"
                if filtered_output.strip():
                    result += filtered_output
                return [TextContent(type="text", text=result)]

        except Exception as e:
            # 捕获执行过程中的异常
            return [
                TextContent(
                    type="text", text=f"<e>Failed to execute maven test: {str(e)}</e>"
                )
            ]
            
def extract_important_maven_output(output):
    """
    提取 Maven 输出中的重要信息，过滤掉不必要的日志
    如果输出少于200行，直接返回全部内容
    否则只对最后1000行应用过滤逻辑
    """
    # 分割输出为行
    output_lines = output.splitlines()
    
    # 如果输出少于200行，直接返回全部内容
    if len(output_lines) <= 200:
        return output
    
    # 如果输出很长，只处理最后1000行
    if len(output_lines) > 1000:
        lines_to_process = output_lines[-1000:]
    else:
        lines_to_process = output_lines
    
    # 存储过滤后的输出行
    important_lines = []
    
    # Maven 核心相关的关键词
    maven_keywords = [
        "BUILD ", "Tests run:", "Failed tests:", "Errors:", "Results :",
        "[ERROR]", "[WARNING]", "[INFO] BUILD", "[INFO] Total time:", 
        "[INFO] Finished at:", "[INFO] Final Memory:", "Failures:", 
        "There are test failures", "Caused by:", "Exception in thread",
        "java.lang.AssertionError", "expected", "Compilation failure",
        "COMPILATION ERROR", "T E S T S"
    ]
    
    # 错误相关堆栈行的识别模式
    stack_pattern = re.compile(r"at [\w$.]+\([\w$.]+\.(java|groovy|kt|scala):\d+\)")
    
    # 追踪错误上下文的标志
    in_error_context = False
    context_lines = 0
    
    for line in lines_to_process:
        # 检查是否包含 Maven 关键词
        if any(keyword in line for keyword in maven_keywords):
            important_lines.append(line)
            in_error_context = True
            context_lines = 5  # 保留接下来的5行作为上下文
            continue
            
        # 检查是否是堆栈跟踪
        if stack_pattern.search(line):
            important_lines.append(line)
            in_error_context = True
            context_lines = 3
            continue
            
        # 保留错误上下文
        if in_error_context and context_lines > 0:
            important_lines.append(line)
            context_lines -= 1
            if context_lines == 0:
                in_error_context = False
    
    # 如果没有找到重要信息，返回一个有用的摘要和指示信息
    if not important_lines:
        return (
            f"Maven output was {len(output_lines)} lines. No detailed error information found in the last 1000 lines.\n"
            "Output may have been truncated. Consider running with more verbose options if needed."
        )
    
    # 添加一个说明，表明这是过滤后的输出
    header = f"Original output was {len(output_lines)} lines. Showing filtered important information:\n{'-' * 80}\n"
    
    return header + "\n".join(important_lines)
